@@ -11,7 +11,7 @@ double int_on_grid(const double& upr,
                    const Rcpp::NumericVector& grid) 
 {
     int m = grid.size();
-    NumericVector tmpvals(4), tmpgrid(4), tmpa(4);
+    NumericVector tmpvals(4), tmpgrid(4), tmpa(4), a(4);
     double uprnew, newint;
     
     double tmpint = 0.0;
@@ -29,15 +29,17 @@ double int_on_grid(const double& upr,
             if (upr < grid[k]) break;
             
             // select length 4 subvectors and calculate spline coefficients
-            tmpvals = NumericVector::create(vals[std::max(k-1, 0)], 
-            vals[k],
-            vals[k+1],
-            vals[std::min(k+2, m-1)]);
-            tmpgrid = NumericVector::create(grid[std::max(k-1, 0)], 
-            grid[k],
-            grid[k+1], 
-            grid[std::min(k+2, m-1)]);
-            tmpa = coef(tmpvals, tmpgrid);
+            tmpvals[0] = vals[std::max(k-1, 0)]; 
+            tmpvals[1] = vals[k];
+            tmpvals[2] = vals[k+1];
+            tmpvals[3] = vals[std::min(k+2, m-1)];
+            
+            tmpgrid[0] = grid[std::max(k-1, 0)]; 
+            tmpgrid[1] = grid[k];
+            tmpgrid[2] = grid[k+1];
+            tmpgrid[3] = grid[std::min(k+2, m-1)];
+            
+            tmpa = coef(tmpvals, tmpgrid, a);
             
             // don't integrate over full cell if upr is in interior
             uprnew = (upr - grid[k]) / (grid[k+1] - grid[k]);
@@ -65,7 +67,7 @@ double inv_int_on_grid(const double& qq,
                        const Rcpp::NumericVector& grid)
 {
     int m = grid.size();
-    NumericVector tmpvals(4), tmpgrid(4), tmpa(4);
+    NumericVector tmpvals(4), tmpgrid(4), tmpa(4), a(4);
     double uprnew, newint, out, qtest;
     double tmpint = 0.0;
     int tmpk = 0;
@@ -83,9 +85,17 @@ double inv_int_on_grid(const double& qq,
     if (q > qtest) {
         for (int k = 1; k < m-2; ++k) {
             // select length 4 subvectors and calculate spline coefficients
-            tmpvals = NumericVector::create(vals[k-1], vals[k], vals[k+1], vals[k+2]);
-            tmpgrid = NumericVector::create(grid[k-1], grid[k], grid[k+1], grid[k+2]);
-            tmpa = coef(tmpvals, tmpgrid);
+            tmpvals[0] = vals[k-1]; 
+            tmpvals[1] = vals[k];
+            tmpvals[2] = vals[k+1];
+            tmpvals[3] = vals[k+2];
+            
+            tmpgrid[0] = grid[k-1]; 
+            tmpgrid[1] = grid[k];
+            tmpgrid[2] = grid[k+1];
+            tmpgrid[3] = grid[k+2];
+            
+            tmpa = coef(tmpvals, tmpgrid, a);
             newint = cubic_integral(0.0, 1.0, tmpa);
             tmpint += newint * (grid[k+1] - grid[k]);
             tmpk = k;
